classdef Agent < GeneralizedAgent
    %AGENT Summary of this class goes here
    %   Detailed explanation goes here
    
    properties (Access = private)
        % variables:
        e_k = [];                   % e(k) = [e_1,..., e_P]'
                                    % formation error, input to the agent's 
                                    % closed loop, 1xdim(model.C) vector
                                    % output for new state, 1xdim(C) vector
        outputOthers = [];          
        ownSavedOutput = [];        % agents saved output
                                    % for conveniency saved in the formation
                                    % memory
        % objects:
        model
    end
    
    methods
        function this = Agent(id,model,output2Coordinates,refInput)
            % call parent class constructor:
            this = this@GeneralizedAgent(id,output2Coordinates,refInput);
            
            % initialize properties:
            this.model = model;
            %this.x_kplus1 = x_0;
            %this.e_k = zeros(size(model.D,2),1);
            this.y_k = this.model.getInitialOutput();
            
            % calculate initial output and coordinates:
            %this.calculateOutput();
            this.calculateCoordinates();
        end % constructor
        function calculateStep(this,formationSpecSwitch)
            this.calculateE(formationSpecSwitch);
            this.y_k = this.model.calculateStep(this.e_k);
            %this.calculateOutput();
            this.calculateCoordinates();
        end % caltulateStep
        function receiveOutputOthers(this,outputOthers,ownSavedOutput)
            this.outputOthers = outputOthers;
            this.ownSavedOutput = ownSavedOutput;
        end % receiveOutputOthers
    end
    methods (Access = private)
        function calculateE(this,formationSpecSwitch)
            numberOfReceivableAgents = sum(~isnan(this.outputOthers),1);
            numberOfReceivableAgents = numberOfReceivableAgents(1);
            if numberOfReceivableAgents > 0
                if strcmp(formationSpecSwitch,'directed Vectors')
                    %add outputs and relative reference input for each agent.
                    %The output vector holds NaN-entries for each agent, from
                    %which can not be received. In adding, the corresponding
                    %reference values for those agents become NaN as well:
                    yPlusR = this.outputOthers + this.refInput;
                    %Replace all NaN entries with 0:
                    yPlusR(isnan(yPlusR)) = 0; 
                    this.e_k = -this.ownSavedOutput + ...
                        1/numberOfReceivableAgents*sum(yPlusR)';
                else
                    direction = zeros(size(this.outputOthers,1),size(this.outputOthers,2));
                    distance = zeros(1,size(this.outputOthers,1));
                    errors = zeros(size(this.outputOthers,1),size(this.outputOthers,2));
                    for l = 1:size(this.outputOthers,1)
                        direction(l,:) = this.outputOthers(l,:) - this.ownSavedOutput';
                        distance(l) =  norm(direction(l,:));
                        errors(l,:) = direction(l,:)/distance(l)*(distance(l) - this.refInput(l));
                    end % for
                    % Replace all NaN entries with 0:
                    errors(isnan(errors)) = 0;
                    % Norm e_k with number of receivable agents:
                    errors = errors/numberOfReceivableAgents;
                    this.e_k = sum(errors)';
                end % if                
            else this.e_k = zeros(size(this.y_k,1),1);
            end
        end % calculateE
        function calculateNewState(this)
            this.x_kplus1 = this.model.A*this.x_kplus1 + this.model.B*this.e_k;
        end % calculateNewState
    end
    methods (Access = protected)
        function calculateOutput(this)
            this.y_k = this.model.C*this.x_kplus1 + this.model.D*this.e_k;
        end % calculateOutput
    end
    
end

