classdef Animator3D < Animator 
    %ANIMATOR3D Summary of this class goes here
    %   Detailed explanation goes here
    
    properties (Access = protected)
    end
    
    methods
        function this = Animator3D(coordinates)
            %figure%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            this = this@Animator(coordinates);
            plot3(coordinates(:,1),...
                coordinates(:,2),...
                coordinates(:,3));
            xlabel('x');
            ylabel('y');
            zlabel('z');
        end % constructor
        
        
        function animate(this,coordinates,adjacency,time,...
                showCommLinksSwitch,showTrailsSwitch,maxTrailLength,...
                showAgentIdSwitch,agentIds,axesDimSwitch,axesDimensionsManual,...
                useObstacles,obstacles)
            cla;
            title(time); % [... ' sec')]
            this.resizeAxes(coordinates,axesDimSwitch,axesDimensionsManual);
            
            % plot virtual leader:
            plot3(coordinates(1,1),coordinates(1,2),coordinates(1,3),...
                this.settingsCommon,this.settingsVirtualLeader);
            
            % plot agents:
            plot3(coordinates(2:end,1),coordinates(2:end,2),...
                coordinates(2:end,3),this.settingsCommon,this.settingsAgents);
            
            % plot trails:
            if showTrailsSwitch
                this.drawTrails(coordinates,maxTrailLength)
            else if ~isempty(this.trailCoordinates.x)
                    this.trailCoordinates.x = [];
                    this.trailCoordinates.y = [];
                    this.trailCoordinates.z = [];
                end % else if
            end % if
            
            % plot agent ids:
            if showAgentIdSwitch
                this.plotAgentIds(coordinates,agentIds)
            end % if
            
            % draw arrows:
            if showCommLinksSwitch
                this.drawArrows(coordinates,adjacency);
            end % if
            
            % draw obstacles:
            if useObstacles
                this.drawObstacles(obstacles);
            end % if
            
            drawnow;
        end % animate
    end
    methods (Access = protected)
        
        
        function determineAxis(this,coordinates)
            %determine the maximum and minimum coordinates for each DoF
            xMax = max(coordinates(:,1)) + 1;
            xMin = min(coordinates(:,1)) - 1;
            yMax = max(coordinates(:,2)) + 1;
            yMin = min(coordinates(:,2)) - 1;
            zMax = max(coordinates(:,3)) + 1;
            zMin = min(coordinates(:,3)) - 1;
            %determine the larger distance between minimum and maximum in
            %x and y to keep uniform space dimensions
            xyDistance = max(xMax - xMin,yMax - yMin);
            axis([((xMax + xMin)/2 - xyDistance/2) ((xMax + xMin)/2 + xyDistance/2)...
                ((yMax + yMin)/2 - xyDistance/2) ((yMax + yMin)/2 + xyDistance/2)...
                zMin zMax]);
        end % determineAxis
        
        
        function resizeAxes(this,coordinates,axesDimSwitch,axesDimensionsManual) %move to animator for both dimensions?
            if strcmp(axesDimSwitch,'auto')
                this.axesDimensions = axis;
                switch length(this.axesDimensions)
                    case 4
                        if max(coordinates(:,1)) > this.axesDimensions(2) ||...
                            min(coordinates(:,1)) < this.axesDimensions(1) ||...
                            max(coordinates(:,2)) > this.axesDimensions(4) ||...
                            min(coordinates(:,2)) < this.axesDimensions(3)
                        this.determineAxis(coordinates);
                        end
                    case 6
                        if max(coordinates(:,1)) > this.axesDimensions(2) ||...
                            min(coordinates(:,1)) < this.axesDimensions(1) ||...
                            max(coordinates(:,2)) > this.axesDimensions(4) ||...
                            min(coordinates(:,2)) < this.axesDimensions(3) ||...
                            max(coordinates(:,3)) > this.axesDimensions(6) ||...
                            min(coordinates(:,3)) < this.axesDimensions(5)
                        this.determineAxis(coordinates);
                        end
                end % switch
            else axis([axesDimensionsManual(1,:) axesDimensionsManual(2,:)...
                    axesDimensionsManual(3,:)])
            end % if
            
        end % resizeAxes
        
        
        function drawArrows(this,coordinates,adjacency)
            relAxesSize = max(this.axesDimensions(2) - this.axesDimensions(1),...
                this.axesDimensions(4) - this.axesDimensions(3))/50;
            for l = 1:length(adjacency)
                try
                    if adjacency(l,1) ~= 0
                        arrow = coordinates(1,:) - coordinates(l,:);
                        normArrow = arrow/norm(arrow);
                        quiver3(coordinates(l,1) + relAxesSize*normArrow(1),...
                            coordinates(l,2) + relAxesSize*normArrow(2),...
                            coordinates(l,3) + relAxesSize*normArrow(3),... %coordinates of agent l
                            arrow(1) - 2*relAxesSize*normArrow(1),... % direction to agent m
                            arrow(2) - 2*relAxesSize*normArrow(2),...
                            arrow(3) - 2*relAxesSize*normArrow(3),...
                            0,this.arrowsLinespecVL,'LineWidth',1,...
                            'MaxHeadSize',0.2);
                    end % end if
                catch me
                end %try
                for m = 2:length(adjacency)
                    try
                        if adjacency(l,m) ~= 0
                            arrow = coordinates(m,:) - coordinates(l,:);
                            normArrow = arrow/norm(arrow);
                            quiver3(coordinates(l,1) + relAxesSize*normArrow(1),...
                                coordinates(l,2) + relAxesSize*normArrow(2),...
                                coordinates(l,3) + relAxesSize*normArrow(3),... %coordinates of agent l
                                arrow(1) - 2*relAxesSize*normArrow(1),... % direction to agent m
                                arrow(2) - 2*relAxesSize*normArrow(2),...
                                arrow(3) - 2*relAxesSize*normArrow(3),...
                                0,this.arrowsLinespecAgents,'LineWidth',1,...
                                'MaxHeadSize',0.2);
                        end % end if
                    catch me
                    end %try
                end
            end
        end % drawArrows
        
        
        function drawTrails(this,coordinates,maxTrailLength)
            try
                %save new coordinates in the trail matrices for x,y,z:
                currTrailLength = size(this.trailCoordinates.x,2);
                this.trailCoordinates.x =...
                    [this.trailCoordinates.x coordinates(2:end,1)];
                this.trailCoordinates.y =...
                    [this.trailCoordinates.y coordinates(2:end,2)];
                this.trailCoordinates.z =...
                    [this.trailCoordinates.z coordinates(2:end,3)];
                currTrailLength = currTrailLength + 1;
                
                %remove all columns which make the matrix longer than 
                %trailLength specifies:
                if currTrailLength > maxTrailLength
                    this.trailCoordinates.x = this.trailCoordinates.x(:,...
                        (currTrailLength - maxTrailLength + 1):end);
                    this.trailCoordinates.y = this.trailCoordinates.y(:,...
                        (currTrailLength - maxTrailLength + 1):end);
                    this.trailCoordinates.z = this.trailCoordinates.z(:,...
                        (currTrailLength - maxTrailLength + 1):end);
                end
                %plot trails:
                plot3(this.trailCoordinates.x,this.trailCoordinates.y,...
                    this.trailCoordinates.z,this.settingsTrails);
            catch me %if the number of agents has changed and coordinate
                %dimension mismatch happens
                this.trailCoordinates = struct('x',[],'y',[],'z',[]);
            end
        end % drawTrails
        
        
        function plotAgentIds(this,coordinates,agentIds)
            axesSizeX = this.axesDimensions(2) - this.axesDimensions(1);
            axesSizeY = this.axesDimensions(4) - this.axesDimensions(3);
            if length(this.axesDimensions) > 4
                axesSizeZ = this.axesDimensions(6) - this.axesDimensions(5);
            else axesSizeZ = 0;
            end % if
            for l = 1:length(agentIds)
                text(coordinates(l,1) + 0.05*axesSizeX,coordinates(l,2) + ...
                    0.05*axesSizeY,coordinates(l,3) + ...
                    0.05*axesSizeZ,num2str(agentIds(l)-1));
            end % for
        end % plotAgentIds
        
        
        function drawObstacles(this,obstacles)
            for l = 1:length(obstacles)
                switch lower(obstacles{l}.type)
                    case 'ball'
                        if isfield(obstacles{l},'faces')        % specify faces
                            faces = obstacles{l}.faces;
                        else
                            faces = 6;
                        end
                        [X,Y,Z] = sphere(faces);
                        X = X*obstacles{l}.radius + obstacles{l}.center(1);
                        Y = Y*obstacles{l}.radius + obstacles{l}.center(2);
                        Z = Z*obstacles{l}.radius + obstacles{l}.center(3);
                        surf(X,Y,Z);   % mesh or surf
                        % mesh(X,Y,Z,zeros(faces));   % mesh or surf
                    case 'wall'
                        point1 = obstacles{l}.corner;
                        point2 = point1 + obstacles{l}.d1;
                        point3 = point2 + obstacles{l}.d2;
                        point4 = point1 + obstacles{l}.d2;
                        % plot obstacle:
                        patch([point1(1) point2(1) point3(1) point4(1)],...
                            [point1(2) point2(2) point3(2) point4(2)],...
                            [point1(3) point2(3) point3(3) point4(3)],...
                            this.obstacleColorspec);
                end
            end % for
        end % drawObstacles
    end
    
end

