classdef QuadrocopterModel < Model
    %MODEL Summary of this class goes here
    %   Detailed explanation goes here

    properties (Access = private)
        controllerModel             % state space model
        % v(k+1)=A*v(k)+B*y_cl(k)
        %   u(k)=C*v(k)+D*y_cl(k)
        x                           % copter states
        t                           % total elapsed time
    end

    methods
        function this = QuadrocopterModel(controllerModel,x_0,y_0)
            this = this@Model();

            this.t = 0;

            this.controllerModel = controllerModel;

            this.x_kplus1 = x_0; % copter states
            this.x = x_0;
            this.v_kplus1 = zeros(size(this.controllerModel.A,1),1);   % vector states
            this.u_k = zeros(size(this.controllerModel.C,1),1);   % control inputs

            %this.y_k = zeros(size(this.plantModel.C,1),1);
            %this.outputDim = length(y_0);
            this.y_0 = y_0;
        end % constructor
        function y_0 = getInitialOutput(this)
            y_0 = this.y_0;
        end % getInitialOutput
        function t_s = getSamplingTime(this)
            t_s = this.controllerModel.Ts;
        end % getSamplingTime
        function output = calculateStep(this,e_k)

            [tr,xr] = ode45(@copter_nonlinear,this.t+[0 this.controllerModel.Ts],this.x,[],this.u_k);
            this.x_kplus1 = xr(end,:)';

            % Compute next control signal:
            this.u_k = this.controllerModel.C *this.v_kplus1 + this.controllerModel.D *[e_k; -this.x];
            this.v_kplus1 = this.controllerModel.A *this.v_kplus1 + this.controllerModel.B *[e_k; -this.x];

            output = this.x([1 3 5]) + this.y_0;

            this.t = this.t + this.controllerModel.Ts;
            this.x = this.x_kplus1;

            function x_new = copter_nonlinear(t,x,u)

                g = 9.81;
                m = 0.640;

                u(1) = u(1) + m*g;

                x_new = [
                    x(2)
                    -u(1)/m * sin(x(9));
                    x(4);
                    u(1)/m * cos(x(9)) * sin(x(11));
                    x(6);
                    u(1)/m * cos(x(9)) * cos(x(11)) - g;
                    x(8);
                    u(2);
                    x(10);
                    u(3);
                    x(12);
                    u(4)];

            end
        end % calculateStep
        % compute the update of the quad-rotor position

    end

end

