classdef RefInputGenerator < handle
    % REFINPUTGENERATOR Converts an aboslute position reference to relative
    % one for each agent. That is, computes what distance should each agent
    % preserve towards each other agent (if it communicates with it)
    
    properties
    end
    
    methods 
        function this = RefInputGenerator()
        end % constructor
        
        % Calculation in case vectors are provided (distance+direction)
        function relRefInput = calculateRelRefInputVectors(this,absRefInput)
            % absRefInput = [r_1,...,r_N]'
            relRefInput = zeros(size(absRefInput,1) + 1,size(absRefInput,2),...
                size(absRefInput,1));
            for l = 1:size(absRefInput,1)
                % first entry is the agents relative position from the
                % virtual leader:
                relRefInput(1,:,l) = absRefInput(l,:);
                % calculate relative distances to the other agents:
                for m = 1:size(absRefInput,1)
                    relRefInput(m + 1,:,l) = absRefInput(l,:) - absRefInput(m,:);
                end % for
            end % for
        end % relRefInputSpatial
        
        % Calulation if only distances are provided
        function relRefInput = calculateRelRefInputDistance(this,absRefInput,formationRefInput)
            % first dimension: distance to each agent (zero for own
            % position
            relRefInput = zeros(size(absRefInput,1) + 1,size(absRefInput,1));
            for l = 1:size(absRefInput,1)
                % first entry is the agents relative position from the
                % virtual leader:
                relRefInput(1,l) = norm(absRefInput(l,:) - formationRefInput');
                for m = 1:size(absRefInput,1)
                    relRefInput(m + 1,l) = norm(absRefInput(l,:) - absRefInput(m,:));
                end % for
            end % for
        end % calculateRelRefInputDistance
    end % methods
    
end % RefInputGenerator

