function varargout = FAST(varargin)
% FAST Starts the GUI of FormAtion Simulation Tool
% 
% Simply evoke FAST to start the GUI.
% Currently the GUI requires preliminary definition of the multi-agent
% system/formation to be simulated via MAT files.
% For details on defining the agents and the scenario to be simulated,
% please refer to the used manual FAST.pdf, which can be found in the \doc
% folder of your FAST installation.
%
% FAST is distributed under GPL license (see code for details).
% Original authors information:
% Andrey Popov          andrey.popov@tu-harburg.de      ICS, TUHH

% Copyright 2010   Andrey Popov, Institute of Control Systems
%                                Hamburg University of Technology
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.


% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @FAST_OpeningFcn, ...
                   'gui_OutputFcn',  @FAST_OutputFcn, ...
                   'gui_LayoutFcn',  [], ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
   gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end

end % Start
% End initialization code - DO NOT EDIT


%% --- Executes just before FAST is made visible.
function FAST_OpeningFcn(hObject, eventdata, handles, varargin)

% This function has no output args, see OutputFcn.

% varargin   unrecognized PropertyName/PropertyValue pairs from the
%            command line (see VARARGIN)

% Choose default command line output for FAST
handles.output = hObject;

% create fields for settings:
if ~isfield(handles,'simulationSettings')
    handles.simulationSettings = struct();
end
if ~isfield(handles,'formationSettings')
    handles.formationSettings = struct();
end
if ~isfield(handles,'changeSettings')
    handles.changeSettings = struct('newChanges',true,'formationChange','none');
end
if ~isfield(handles,'animationSettings')
    handles.animationSettings = struct();
end

% Update handles structure
guidata(hObject, handles);

% suppress warnings for equation-solving if obstacles are present:
% warning off

% UIWAIT makes FAST wait for user response (see UIRESUME)
% uiwait(handles.figure1);
end % FAST_OpeningFcn


%% --- Outputs from this function are returned to the command line.
function varargout = FAST_OutputFcn(hObject, eventdata, handles)
% varargout  cell array for returning output args (see VARARGOUT);

% Get default command line output from handles structure
varargout{1} = handles.output;
end % FAST_OutputFcn


%% Main Buttons
%##########################################################################
% MAIN BUTTONS
%##########################################################################
function newSimulation_Callback(hObject, eventdata, handles)

% enable and disable buttons:
set(handles.startSimulation,'String','Start Simulation');
set(handles.startSimulation,'Enable','on');
set(handles.plot,'Enable','off');
set(handles.saveData,'Enable','off');
set(handles.formationSpecSwitch,'Enable','off');

% store settings:
handles.formationSettings = storeFormationSettings(hObject,handles);
handles.animationSettings = storeAnimationSettings(hObject,handles);

% read initialization Data:
handles.initializationSettings = storeInitializationData(hObject,handles);

% set 'interrupt' and 'finish':
handles.simulationSettings.interrupt = false;
handles.simulationSettings.finish = true;

% Create new simulation:
settings = struct(...
    'initialization',handles.initializationSettings,...
    'formation',handles.formationSettings,...
    'changes',handles.changeSettings,...
    'animation',handles.animationSettings,...
    'simulation',handles.simulationSettings);

if isstruct(handles.initializationSettings)
    handles.simulation = SimulationManager(settings);
    guidata(hObject, handles);
    % initialize the simulation:
    handles.simulation.initialization();
end % if

end % newSimulation_Callback



function startSimulation_Callback(hObject, eventdata, handles)

% distinguish different button cases:
switch get(handles.startSimulation,'String')
    case 'Start Simulation'
        
        % change and enable/disable buttons:
        set(handles.startSimulation,    'String','Interrupt Simulation');
        set(handles.newSimulation,      'Enable','off');
        set(handles.endSimulation,      'Enable','on');
        set(handles.plot,               'Enable','off');
        set(handles.saveData,           'Enable','off');
        set(handles.refInputSwitch,     'Enable','off');
        set(handles.refInputInput,      'Enable','off');
        set(handles.timedelaySwitch,    'Enable','off');
        set(handles.timedelayInput,     'Enable','off');
        set(handles.timedelayFormatSwitch,'Enable','off');
        set(handles.useOwnTimeDelay,    'Enable','off');
        set(handles.useLimitedCommRange,'Enable','off');
        set(handles.commRangeInput,     'Enable','off');
        set(handles.useObstacles,       'Enable','off');
        set(handles.useChanges,         'Enable','off');
        
        % store settings:
        handles.formationSettings = storeFormationSettings(hObject,handles);
        
        % set 'interrupt' and 'finish':
        handles.simulation.setInterrupt(false);
        handles.simulation.setFinish(false);
        
        % fast simulation:
        handles.simulation.simulate();
        
    case 'Interrupt Simulation'
        % change and enable/disable buttons:
        set(handles.startSimulation,    'String','Resume Simulation');
        set(handles.plot,               'Enable','on');
        set(handles.saveData,           'Enable','on');
        set(handles.refInputSwitch,     'Enable','on');
        if get(handles.refInputSwitch,  'Value') == 1
            set(handles.refInputInput,  'Enable','on');
        end
        set(handles.timedelaySwitch,    'Enable','on');
        if get(handles.timedelaySwitch, 'Value') == 2
            set(handles.timedelayInput, 'Enable','on');
            set(handles.timedelayFormatSwitch,'Enable','on');
            set(handles.useOwnTimeDelay,'Enable','on');
        end
        set(handles.useLimitedCommRange,'Enable','on');
        if get(handles.useLimitedCommRange,'Value') == 1
            set(handles.commRangeInput,'Enable','on');
        end
        set(handles.useObstacles,       'Enable','on');
        set(handles.useChanges,         'Enable','on');
        
        % set 'interrupt' and 'finish':
        handles.simulation.setInterrupt(true);
        
    case 'Resume Simulation'
        % Disable buttons during simulation
        set(handles.startSimulation,'String','Interrupt Simulation');
        set(handles.plot,           'Enable','off');
        set(handles.saveData,       'Enable','off');
        set(handles.refInputSwitch, 'Enable','off');
        set(handles.refInputInput,  'Enable','off');
        set(handles.timedelaySwitch,'Enable','off');
        set(handles.timedelayInput, 'Enable','off');
        set(handles.timedelayFormatSwitch,'Enable','off');
        set(handles.useOwnTimeDelay,'Enable','off');
        set(handles.useLimitedCommRange,'Enable','off');
        set(handles.commRangeInput, 'Enable','off');
        set(handles.useObstacles,   'Enable','off');
        set(handles.useChanges,     'Enable','off');
        
        % set 'interrupt' and 'finish':
        handles.simulation.setInterrupt(false);
        
        % store settings:
        handles.formationSettings = storeFormationSettings(hObject,handles);
end % switch
end % startSimulation_Callback


function endSimulation_Callback(hObject, eventdata, handles)

% change and enable buttons:
set(handles.startSimulation,        'String','Start Simulation');
set(handles.startSimulation,        'Enable','off');
set(handles.endSimulation,          'Enable','off');
set(handles.newSimulation,          'Enable','on');
set(handles.plot,                   'Enable','on');
set(handles.saveData,               'Enable','on');

set(handles.refInputSwitch,         'Enable','on');
if get(handles.refInputSwitch,      'Value') == 1
    set(handles.refInputInput,      'Enable','on');
end
set(handles.timedelaySwitch,        'Enable','on');
if get(handles.timedelaySwitch,     'Value') == 2
    set(handles.timedelayInput,     'Enable','on');
    set(handles.timedelayFormatSwitch,'Enable','on');
    set(handles.useOwnTimeDelay,    'Enable','on');
end
set(handles.useLimitedCommRange,    'Enable','on');
if get(handles.useLimitedCommRange, 'Value') == 1
    set(handles.commRangeInput,     'Enable','on');
end
set(handles.useObstacles,           'Enable','on');
set(handles.useChanges,             'Enable','on');
set(handles.formationSpecSwitch,    'Enable','on');

% set 'interrupt' and 'finish':
handles.simulation.setInterrupt(false);
handles.simulation.setFinish(true);

end % endSimulation_Callback



function plot_Callback(hObject, eventdata, handles)
% dialog box to ask for plotting:
%default = 'Yes';
choice{1} = 'x-y';
choice{2} = 'x-z';
choice{3} = 'y-z';
choice{4} = 't-x';
choice{5} = 't-y';
choice{6} = 't-z';

plotChoice = listdlg('ListString',choice,'Name','Plot 2-D views',...
    'ListSize',[150 150],'PromptString','Please choose views!');

handles.simulation.plot(plotChoice);
end % plot_Callback



function saveData_Callback(hObject, eventdata, handles)
[filename,pathname] = uiputfile('output\simulationData.mat',...
    'Save Simulation Data');
handles.simulation.saveData(pathname,filename);
end % saveData_Callback




%% Initialization
%##########################################################################
% INITIALIZATION
%##########################################################################
function initializationSettings = storeInitializationData(hObject,handles)
% read initialization data from file:
try
    [initFileName,initPathName] = uigetfile('input\initialization.mat',...
    'Please select an initialization file');
catch me
end % try
if initFileName ~= 0
    initializationData = load([initPathName initFileName]);
    initializationSettings.models = initializationData.models;
    initializationSettings.output2Coordinates = initializationData.output2Coordinates;
    initializationSettings.adjacency = initializationData.adjacency;
    initializationSettings.absRefInput = initializationData.absRefInput;
else %errordlg('No initialization data chosen! Please start a new simulation!','Error');
    set(handles.startSimulation,'Enable','off');
    initializationSettings = [];
end % if

% save handle structure
guidata(hObject,handles);

% save settings in SimulationManager, if simulation already created:
if isfield(handles,'simulation')
    handles.simulation.setInitializationSettings(initializationSettings);
end

end % storeInitializationData




%% Formation Settings
%##########################################################################
% FORMATION SETTINGS
%##########################################################################
function formationSettings = storeFormationSettings(hObject,handles)

formationSettings = handles.formationSettings;
% store all formation settings:

if ~isfield(formationSettings,'referenceFile')
    formationSettings.referenceFile = false;
end
% Manual formation reference input:
if ~formationSettings.referenceFile
    formationSettings.formationRefInput.refPos = cell2mat(textscan(get...
        (handles.refInputInput,'String'),'%n'));
end

% formation specification:
if get(handles.formationSpecSwitch,'Value') == 1
    formationSettings.formationSpecSwitch = 'directed Vectors';
else formationSettings.formationSpecSwitch = 'Distances';
end % if
    
% communication time delay:
switch get(handles.timedelaySwitch,'Value')
    case 1
        formationSettings.timedelaySwitch.Type = 'none';
        formationSettings.timedelay = NaN;
    case 2
        formationSettings.timedelaySwitch.Type = 'fixed';
        formationSettings.timedelaySwitch.DelayOwn = get(handles.useOwnTimeDelay,'Value');
        if get(handles.timedelayFormatSwitch,'Value') == 1
            formationSettings.timedelaySwitch.DelayUnits = 'steps';
            formationSettings.timedelay = max(0,...
                floor(str2double(get(handles.timedelayInput,'String'))));
        else
            formationSettings.timedelaySwitch.DelayUnits = 'sec.';
            formationSettings.timedelay = ...
                max(0,str2double(get(handles.timedelayInput,'String')));
        end
end
    
% communication range:
if get(handles.useLimitedCommRange,'Value') == 0
    formationSettings.useCommRangeLimit = false;
    formationSettings.maxCommunicationDist = NaN;
else
    formationSettings.useCommRangeLimit = true;
    formationSettings.maxCommunicationDist = str2double(get...
        (handles.commRangeInput,'String'));
end

% communication obstacles:
if get(handles.useObstacles,'Value') == 0
    formationSettings.useObstacles = false;
    formationSettings.obstacles = NaN;
else
    formationSettings.useObstacles = true;
end

% changes obstacles:
if get(handles.useChanges,'Value') == 0
    handles.changeSettings.formationChange = 'none';
end

% save settings in SimulationManager, if simulation already created:
if isfield(handles,'simulation')
    if handles.changeSettings.newChanges
        handles.simulation.setChangeSettings(handles.changeSettings);
        handles.changeSettings.newChanges = false;
    end
    handles.simulation.setFormationSettings(formationSettings);
end

% save handle structure
guidata(hObject,handles);

end % storeFormationSettings


%% Reference Input
%##########################################################################
% REFERENCE INPUT
%##########################################################################
function refInputSwitch_Callback(hObject, eventdata, handles)
% Hints: contents = get(hObject,'String') returns refInputSwitch contents as cell array
%        contents{get(hObject,'Value')} returns selected item from refInputSwitch

referenceFile = false;
if get(handles.refInputSwitch,'Value') == 2
    [initFileName,initPathName] = uigetfile(get(handles.fileReference,'String'),...
        'Please select a reference input file');
    try
        if initFileName ~= 0
            refInputData = load(fullfile(initPathName,initFileName));
            handles.formationSettings.formationRefInput = refInputData.formationRefInput;
            % add zero coordinates at time zero, because the user might
            % have forgotten it or intentionally left empty
            if isfield(handles.formationSettings.formationRefInput, 't_step') ...
                    && ~isempty(handles.formationSettings.formationRefInput.t_step)
                handles.formationSettings.formationRefInput.t_step = [0; handles.formationSettings.formationRefInput.t_step(:)];
                handles.formationSettings.formationRefInput.refPos = [[0;0;0], handles.formationSettings.formationRefInput.refPos];
            end                
            set(handles.fileReference,'String',initFileName);
            referenceFile = true;
        else
        end
    catch me
    end
end
if referenceFile
    handles.formationSettings.referenceFile = true;
    set(handles.refInputInput,'Enable','off')
    set(handles.refInputInput,'Visible','off');
    set(handles.fileReference,'Visible','on');
else
    handles.formationSettings.referenceFile = false;
    set(handles.refInputSwitch,'Value', 1);
    set(handles.refInputInput,'Visible','on');
    set(handles.refInputInput,'Enable','on');
    set(handles.fileReference,'Visible','off');
end
guidata(hObject,handles);
end % refInputSwitch_Callback


function refInputSwitch_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end % if
end % refInputSwitch_CreateFcn


function refInputInput_Callback(hObject, eventdata, handles)
end % refInputInput_Callback


function refInputInput_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end % if
end % refInputInput_CreateFcn


function fileReference_Callback(hObject, eventdata, handles)
end


function fileReference_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
end




%% Formation Specification
%##########################################################################
% FORMATION SPECIFICATION
%##########################################################################
function formationSpecSwitch_Callback(hObject, eventdata, handles)
end % formationSpecSwitch

function formationSpecSwitch_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end % if
end % formationSpeckSwitch_CreateFcn


%% Time Delay
%##########################################################################
% TIME DELAY
%##########################################################################
function timedelayInput_Callback(hObject, eventdata, handles)
end % timedelayInput_Callback

function timedelayInput_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end % if
end % timedelayInput_CreateFcn


function timedelaySwitch_Callback(hObject, eventdata, handles)
if get(handles.timedelaySwitch,'Value') == 1
    set(handles.timedelayInput,'Enable','off');
    set(handles.timedelayFormatSwitch,'Enable','off');
    set(handles.useOwnTimeDelay,'Enable','off');
else
    set(handles.timedelayInput,'Enable','on');
    set(handles.timedelayFormatSwitch,'Enable','on');
    set(handles.useOwnTimeDelay,'Enable','on');
end % if
end % timedelaySwitch_Callback


function timedelaySwitch_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end % if
end % timedelaySwitch_CreateFcn

function timedelayFormatSwitch_Callback(hObject, eventdata, handles)
end


function timedelayFormatSwitch_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
end


function useOwnTimeDelay_Callback(hObject, eventdata, handles)
end


%% Communication Range
%##########################################################################
% COMMUNICATION RANGE
%##########################################################################
function commRangeInput_Callback(hObject, eventdata, handles)
end % commRangeInput_Callback


function commRangeInput_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end % if
end % commRangeInput_CreateFcn


function useLimitedCommRange_Callback(hObject, eventdata, handles)
if get(handles.useLimitedCommRange,'Value') == 0
    set(handles.commRangeInput,'Enable','off');
else set(handles.commRangeInput,'Enable','on');
end % if
guidata(hObject,handles);
end



%% Communication Obstacles
%##########################################################################
% COMMUNICATION OBSTACLES
%##########################################################################
function commObstaclesSwitch_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end % if
end % commObstaclesSwitch_CreateFcn


function useObstacles_Callback(hObject, eventdata, handles)
obstaclesSet = false;
if get(handles.useObstacles,'Value') == 1
    [initFileName,initPathName] = uigetfile(get(handles.fileObstacles,'String'),...
        'Please select an obstacle file');
    try
        if initFileName ~= 0
            obstacleData = load(fullfile(initPathName,initFileName));
            handles.formationSettings.useObstacles = true;
            handles.formationSettings.obstacles = obstacleData.obstacles;
            set(handles.fileObstacles,'String',initFileName);
%             set(handles.fileObstacles,'Enable', 'on');
            set(handles.fileObstacles,'Visible', 'on');
            obstaclesSet = true;
        end
    catch me
    end
end
if ~obstaclesSet
    set(handles.useObstacles,'Value',0);
%     set(handles.fileObstacles,'Enable', 'off');
    set(handles.fileObstacles,'Visible', 'off');
    handles.formationSettings.useObstacles = false;
    handles.formationSettings.obstacles = NaN;
end % if
guidata(hObject,handles);
end


function fileObstacles_Callback(hObject, eventdata, handles)
end


function fileObstacles_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
end


%% Changes to the formation/multi-agent system
%##########################################################################
% FORMATION CHANGES
%##########################################################################
function useChanges_Callback(hObject, eventdata, handles)
changeSettings = handles.changeSettings;

% formation change data:
changesFile = false;
if get(handles.useChanges,'Value')
    [initFileName,initPathName] = uigetfile(get(handles.fileChanges,'String'),...
        'Please select a change data file');
    try
        if initFileName ~= 0
            changeData = load(fullfile(initPathName,initFileName));
            changeSettings.changeData = changeData.changeData;
            set(handles.fileChanges,'String',initFileName);
            set(handles.fileChanges,'Visible','on');
            changesFile = true;
        end % if
    catch me
    end
end % if

if changesFile
    set(handles.fileChanges,'Visible','on');
    changeSettings.formationChange = 'from file';
    handles.changeSettings.newChanges = true;
else
    set(handles.fileChanges,'Visible','off');
    set(handles.useChanges,'Value',0);
    changeSettings.formationChange = 'none';
    changeSettings.changeData = [];
end

handles.changeSettings = changeSettings;
guidata(hObject,handles);   % save handle structure
end % useChanges_Callback


function fileChanges_Callback(hObject, eventdata, handles)
end


function fileChanges_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
end



%% Animation functions
%##########################################################################
% ANIMATION SETTINGS
%##########################################################################

function showCommLinksSwitch_Callback(hObject, eventdata, handles)
handles.animationSettings = storeAnimationSettings(hObject,handles);
end % showCommLinksSwitch_Callback


function showAgentIdSwitch_Callback(hObject, eventdata, handles)
handles.animationSettings = storeAnimationSettings(hObject,handles);
end % showAgentIdSwitch_Callback


function showTrailsSwitch_Callback(hObject, eventdata, handles)
handles.animationSettings = storeAnimationSettings(hObject,handles);
end % showTrailsSwitch_Callback


function trailLengthInput_Callback(hObject, eventdata, handles)
handles.animationSettings = storeAnimationSettings(hObject,handles);
end % trailLengthInput_Callback


function trailLengthInput_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end % if
end % trailLengthInput_CreateFcn


function animationRateInput_Callback(hObject, eventdata, handles)
input = round(str2double(get(handles.animationRateInput,'String')));
if input < 1
    input = 1;
end % if
set(handles.animationRateInput,'String',input);
handles.animationSettings = storeAnimationSettings(hObject,handles);
end % animationRateInput_Callback


function animationRateInput_CreateFcn(hObject, eventdata, handles)
if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor',[.9 .9 .9]);
end % if
end % animationRateInput_CreateFcn


function axesDimSwitch_Callback(hObject, eventdata, handles)
if get(handles.axesDimSwitch,'Value') == 1
    set(handles.axesDimInput,'Enable','off');
else set(handles.axesDimInput,'Enable','on');
end % if
handles.animationSettings = storeAnimationSettings(hObject,handles);
end % axesDimSwitch_Callback


function axesDimSwitch_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end % if
end % axesDimSwitch_CreateFcn


function axesDimInput_CellEditCallback(hObject, eventdata, handles)
% hObject    handle to axesDimInput (see GCBO)
% eventdata  structure with the following fields (see UITABLE)
%	Indices: row and column indices of the cell(s) edited
%	PreviousData: previous data for the cell(s) edited
%	EditData: string(s) entered by the user
%	NewData: EditData or its converted form set on the Data property. Empty if Data was not changed
%	Error: error string when failed to convert EditData to appropriate value for Data
% handles    structure with handles and user data (see GUIDATA)

% make sure that the lower value for axes dimensions is smaller than the
% upper value, otherwise keep the previous value:
data = get(handles.axesDimInput,'Data');
if eventdata.Indices(2) == 2 && eventdata.NewData <= data(...
        eventdata.Indices(1),1) ||...
        eventdata.Indices(2) == 1 && eventdata.NewData >= data(...
        eventdata.Indices(1),2)
    data(eventdata.Indices(1),eventdata.Indices(2)) = eventdata.PreviousData;
    set(handles.axesDimInput,'Data',data);
    set(handles.axesDimWarning,'Visible','on');
else handles.animationSettings = storeAnimationSettings(hObject,handles);
    set(handles.axesDimWarning,'Visible','off');
end % if
end % axesDimInput_CellEditCallback


function animationSettings = storeAnimationSettings(hObject, handles)
animationSettings = handles.animationSettings;
% store all animation settings:

% communication links:
animationSettings.showCommLinksSwitch = get...
    (handles.showCommLinksSwitch,'Value');

% agent ids:
animationSettings.showAgentIdSwitch = get...
    (handles.showAgentIdSwitch,'Value');

% trails:
animationSettings.showTrailsSwitch = get...
    (handles.showTrailsSwitch,'Value');
if get(handles.showTrailsSwitch,'Value') == 1.0
    animationSettings.trailLength = str2double(get...
        (handles.trailLengthInput,'String'));
else animationSettings.trailLength = NaN;
end

% animation rate:
animationSettings.animationRate = str2double(get...
    (handles.animationRateInput,'String'));

% axes dimensions:
if get(handles.axesDimSwitch,'Value') == 1
    animationSettings.axesDimSwitch = 'auto';
    animationSettings.axesDimensions = [];
else animationSettings.axesDimSwitch = 'manual';
    animationSettings.axesDimensions = get(handles.axesDimInput,'Data');
end

% save handle structure
guidata(hObject,handles);

% save settings in SimulationManager, if simulation already created:
if isfield(handles,'simulation')
    handles.simulation.setAnimationSettings(animationSettings);
end

end % storeAnimationSettings



%##########################################################################
% TOOLBAR BUTTONS
%##########################################################################
function view_xy_ClickedCallback(hObject, eventdata, handles)
try
    view(0,90);
catch me
end % try
end % view_xy_ClickedCallback


function view_xz_ClickedCallback(hObject, eventdata, handles)
try
    view(0,0);
catch me
end % try
end % view_xz_ClickedCallback


function view_yz_ClickedCallback(hObject, eventdata, handles)
try
    view(90,0);
catch me
end % try
end % view_yz_ClickedCallback


function view_3d_ClickedCallback(hObject, eventdata, handles)
try
    view(3);
catch me
end % try
end % view_3d_ClickedCallback
