%% Specify Agents and References
% 1. Load the agent and controller models
load copter_models P K  % plant P and controller K

% 2. Specify the agents        init. state  init. position
models = [];
models{1} = LinearModel( P, K, zeros(12,1), [1 0 0]' );
models{2} = LinearModel( P, K, zeros(12,1), [2 0 0]' );
models{3} = LinearModel( P, K, zeros(12,1), [3 0 0]' );
models{4} = LinearModel( P, K, zeros(12,1), [4 0 0]' );
models{5} = LinearModel( P, K, zeros(12,1), [5 0 0]' );

% 3. Spatial Coordinates
output2Coordinates = [
    1 0 0;
    0 1 0;
    0 0 1];

% 4. Adjacency matrix
%    agent #  0 1 2 3 4 5
adjacency = [0 0 0 0 0 0; % this is the Reference point
             -1 0 1 1 0 0; % agent 1: receives from ref. point, agents 2 & 3
             -1 0 0 0 1 0; % agent 2: receives from ref. point, agent 4
             0 1 0 0 1 0; % agent 3: receives from agents 1 & 4
             0 0 1 1 0 0; % agent 4: receives from agents 2 & 3
             0 0 1 0 1 0];% agent 5: receives from agents 2 & 4

% 5. Absolute reference input for all agents:
absRefInput = [
    0   -1  0;    % agent 1
    -1  -2  0;    % agent 2
    1   -2  0;
    -1  -3  0;
    1   -3  0];   % agent 5

% Save to initialization.mat file
save Janes_init.mat   models output2Coordinates adjacency absRefInput;



%% Specify obstacles
% Ball-shaped obstacle, where 'faces' is used for visualization
obstacles{1} = struct('type','ball','center',[0 1 1],'radius',0.5,'faces',6);
% Wall-shaped obstacle with corners: corner -- corner+d1 -- corner+d1+d2 -- corner+d2
obstacles{2} = struct('type','wall','corner',[3 0 -1],'d1',[0 0 2],'d2',[0 2 0]);

% Save the obstacles to obstacles.mat
save Janes_obstacles.mat obstacles



%% Specify reference signal
t = 0:1000;            % time-steps
x = 1 * sin(0.01*t);   % x-axis
y = 0.02 * t;          % y-movement with speed of 0.02 units per time-step
z = ones(1,length(t)); % constant z-position
formationRefInput = struct('refPos', [x; y; z]);

save Janes_reference.mat formationRefInput



%% Specify topology changes
changeData = [];
changeData{1} = struct(...
    'type',       'commTopology',...
    't_change',   100,  ...   % time step of change
    'adjacency',  [
     0 0 0 0 0 0;
    -1 0 1 1 0 0;
     0 1 0 0 1 0;
     0 1 0 0 1 0;
     0 0 1 1 0 0;
     0 0 1 0 1 0]);

 changeData{2} = struct(...
    'type',       'commTopology',...
    't_change',   200,  ...   % step at which the change should occur
    'adjacency',  [
     0 0 0 0 0 0;
    -1 0 1 1 1 1;
     0 1 0 1 1 1;
     0 1 1 0 1 1;
     0 1 1 1 0 1;
     0 1 1 1 1 0]);
 
 
 changeData{3} = struct(...
     'type',       'constellation',...
     't_change',   250, ...
     'absRefInput', [
     0  -1    1;    % agent 1  [x y z] - coordinates
     0  -1.5  1;    % agent 2
     0  -2    1;
     0  -2.5  1;
     0  -3    1]);
 
 load copter_models K P  % controller K and plant P
 new_agents = [];
 new_agents{1} = LinearModel( P, K, zeros(12,1), [-5 0 0]' );
 new_agents{2} = LinearModel( P, K, zeros(12,1), [-6 0 0]' );
 
 changeData{4} = struct(...
     'type',            'addAgent',...
     't_change',        300, ...
     'newAbsRefInput', [-2 -4 1; 2 -4 1],... % with respect to virtual ref.
     'adjacency', [
     0 0 0 0 0 0 0 0;
    -1 0 1 1 1 1 0 0;
     0 1 0 1 1 1 0 0;
     0 1 1 0 1 1 1 0;
     0 1 1 1 0 1 1 0;
     0 1 1 1 1 0 0 0;
     0 1 0 0 0 0 0 0;
     0 0 0 0 0 1 0 0], ...
     'models', {new_agents});

 changeData{5} = struct(...
     'type',       'deleteAgent',...
     't_change',   700, ...
     'deleteIds', [2 3]);
  
 save Janes_changes.mat changeData
