%% Specify Agents and References
% 1. Load the agent and controller models
load copter_models P K  % plant P and controller K

models = [];
% 2. Specify the agents        init. state  init. position
N = 5;  % num of agents
Init_Pos = [(1:N)', zeros(N,1), zeros(N,1)];
for ind = 1:2     % linear models
    models{ind} = LinearModel( P, K, zeros(12,1), Init_Pos(ind,:)' );
end
for ind = 3:N     % non-linear models
    models{ind} = QuadrocopterModel( K, zeros(12,1), Init_Pos(ind,:)' );
end

% 3. Spatial Coordinates
output2Coordinates = eye(3);    % x-y-z

% 4. Adjacency matrix
%    agent # 0 1 2 3 4 5
adjacency = [0 0 0 0 0 0; % this is the Reference point
            -1 0 1 1 0 0; % agent 1: receives from ref. point, agents 2 & 3
            -1 0 0 0 1 0; % agent 2: receives from ref. point, agent 4
             0 1 0 0 1 0; % agent 3: receives from agents 1 & 4
             0 0 1 1 0 0; % agent 4: receives from agents 2 & 3
             0 0 1 0 1 0];% agent 5: receives from agents 2 & 4

% 5. Absolute reference input for all agents:
absRefInput = [
    0   -1  0;    % agent 1
    -1  -2  0;    % agent 2
    1   -2  0;
    -1  -3  0;
    1   -3  0];   % agent 5


%% Specify obstacles
% Ball-shaped obstacle, where 'faces' is used for visualization
obstacles{1} = struct('type','ball','center',[0 1 1],'radius',0.5,'faces',6);
% Wall-shaped obstacle
obstacles{2} = struct('type','wall','corner',[3 0 -1],...
     'd1',[0 0 2],'d2',[0 2 0]);


%% Specify reference signal
formationRefInput.t_step = [0, 200, 500, 1000];    % ref. positions at time steps
formationRefInput.refPos = [
    0 0 0;          % at 0
    0 2 0;          % at 200
    0 5 1;          % at 500
    0 6 2]';        % at 1000    % note the transpose!
        


%% Specify topology changes
changeData = [];

% Change topology
adj2 = adjacency;
adj2(1,3) = 0;      % remove link 0-->2
adj2(2,3) = 1;      % add link    1-->2
changeData{1} = struct(...
    'type',       'commTopology',...
    't_change',   100,  ...   % time step of change
    'adjacency',  adj2);

% Change topology
adj3 = adj2;
adj3(2:end,2:end) = ones(N); % make all communications
for i=2:N+1
    adj3(i,i) = 0;  % set the diagonal of the adj. to 0
end
changeData{2} = struct(...
    'type',       'commTopology',...
    't_change',   200,  ...   % step at which the change should occur
    'adjacency',  adj3);

% Chenge formation shape
changeData{3} = struct(...
    'type',       'constellation',...
    't_change',   250, ...
    'absRefInput', [
    0  -1    1;    % agent 1  [x y z] - coordinates
    0  -1.5  1;    % agent 2
    0  -2    1;
    0  -2.5  1;
    0  -3    1]);

% Add agents
load copter_models P K  % plant P and controller K
new_agents = [];
new_agents{1} = LinearModel( P, K, zeros(12,1), [-5 0 0]' );
new_agents{2} = LinearModel( P, K, zeros(12,1), [-6 0 0]' );

%                    agents    3  & 4   receive from the new agent
adj4 = [adj3, [0 0; 0 0; 0 0; 1 0; 1 0; 0 0];
    0 1 0 0 0 0 0 0;   % the first new agent receives form agent 1
    0 0 0 0 0 1 0 0];  % the second new agent recevies from agent 5
% #  0 1 2 3 4 5 6 7    - agent number
changeData{4} = struct(...
    'type',            'addAgent',...
    't_change',        300, ...
    'newAbsRefInput', [-2 -4 1; 2 -4 1],... % with respect to virtual ref.
    'adjacency', adj4, ...
    'models', {new_agents});                % note the {} brackets!

% Remove agents
changeData{5} = struct(...
    'type',       'deleteAgent',...
    't_change',   700, ...
    'deleteIds', [2 3]);

% Save everything to Janes_all_sim.mat
save Janes_all_sim.mat